function [outNZ,outNZ1,outNZ2,outNZ3,outNZ4,outReconNLfin,outReconNLSfin,outWaveSUREHfin,outWaveSURESfin,outWaveTIfin,outWaveBayesSfin]=Denoising(nbpoint,profmin,cas,levels) 
 
 %nbpoints: number of points for the computation of H at the finest scale
 %profmin : indicates the minimum depth of decomposition for label determination  
 %cas = 1 : piece-wise cubic, cas = 2: block signal, cas = 3: piece-wise regular 
 %levels  : number of level considered for the wavelet transforms 
 
 %outNZ,outNZi(i=1:4) : contains the average number of nonzero coefficients kept after 
 %                      thresholding the different decomposition 
 %outReconsNLfin      : average SNR associated with the reconstruction using
 %                      nonlinear methods after universal thresholding
 %outReconsNLSfin     : average SNR associated with the reconstruction using 
 %                      nonlinear methods after Bayes thresholding at low SNR
 %outWaveSUREHfin     : average SNR associated with the reconstruction using 
 %                      nonlinear methods after SURE Hard wavelet thresholding
 %outWaveSURESfin     : Same as previously except the soft thresholding is
 %                      used instead
 %outWaveTIfin        : average SNR when the translation invariant wavelet
 %                      threshold is used
 %outWaveBayesSfin    : average SNR when the BayesShrink technique is used
 %                 
 
 %studied signal
 
 if (cas == 1)
   s = makesignal('Piece-Polynomial',4096);
 elseif (cas == 2)   
   s = makesignal('Blocks',4096);
 else
   s = makesignal('Piece-Regular',4096);
 end
 
 L = length(s);
 sigpower =  mean(s.^2);
 
 NZ           = zeros(20,40); %number of nonzero coefficients after thresholding with the nonlinear prediction
 NZ1          = zeros(20,40); %idem but considering the Bayes thresholding when SNR 12
 NZ2          = zeros(20,40); %Number of nonzero coefficients after SURE hard thresholding 
 NZ3          = zeros(20,40); %Number of nonzero coefficients after SURE soft thresholding
 NZ4          = zeros(20,40); %Number of nonzero coefficients after BayesShrink thresholding 
 outReconNLS  = zeros(20,40); %Reconstruction of the signal with the nonlinear prediction using a 
                              %Bayes threshold at low SNR 
 outReconNL   = zeros(20,40); %Reconstruction of the signal with the nonlinear prediction
 outWaveSUREH = zeros(20,40); %Reconstruction of the signal when the SURE hard thresholding is applied 
 outWaveSURES = zeros(20,40); %Reconstruction of the signal when the SURE soft thresholding is applied 
 outWaveTI    = zeros(20,40); %Reconstruction of the signal when the translation invariant wavelet thresholding is applied
 outWaveBayesS= zeros(20,40); %Reconstruction of the signal when the Bayesshrink threshold is applied
 
 for k=1:40, %we average the results over 40 realizations 
  for SNR= 20:-1:1,
   noisepower  = sigpower/(10^(SNR/10));
   noisesignal = randn(1,length(s))*sqrt(noisepower);
   signal      = s + noisesignal; %noisy signal
  
   %label contains the location of the singularities until the depth of 
   %decomposition given by nbpoints
   [label] = labelNew(signal,nbpoint,profmin,nbpoint);
   
   %decomposition of the signal using the detected singularities
   [MS]     = decompose(signal,nbpoint,label);
   
   %computation of the threshold on the finest scale
   sigma = median(abs(MS(L/2+1:L)))/0.6745; 
   
   %thresholding of the coefficients taking into account the normalization of 
   %the coefficients, we consider the bayes thresholding at low SNR 
   
   MSS = MS;
   
   for p=1:nbpoint-1,
    MS(L/(2^p)+1:L/(2^(p-1))) = MS(L/(2^p)+1:L/(2^(p-1))).*...
      (abs(MS(L/(2^p)+1:L/(2^(p-1)))) > sigma/sqrt(2^(p-1))*sqrt(2*log(L)));     

    if SNR > 12
     MSS(L/(2^p)+1:L/(2^(p-1))) = MSS(L/(2^p)+1:L/(2^(p-1))).*...
       (abs(MSS(L/(2^p)+1:L/(2^(p-1)))) > sigma/sqrt(2^(p-1))*sqrt(2*log(L)));     
    else
     threshold = bayes(MSS(L/(2^p)+1:L/(2^(p-1))),sigma/sqrt(2^(p-1)));
     MSS(L/(2^p)+1:L/(2^(p-1))) = sthresh(MSS(L/(2^p)+1:L/(2^(p-1))),threshold);
    end
    
   end
  
   ReconNL  = reconstruct(MS,nbpoint,label);
   %number of nonzero coefficients
   X = MS(L/2^(nbpoint-1)+1:end);
   NZ(SNR,k) = length(X(X~=0));

   ReconNLS  = reconstruct(MSS,nbpoint,label);
   %number of nonzero coefficients
   X = MSS(L/2^(nbpoint-1)+1:end);
   NZ1(SNR,k) = length(X(X~=0));

   %wavelet denoising using SURE hard thresholding  
   [Wave_SUREH,C,R] = wden(signal,'rigrsure','h','sln',levels,'sym4');
   X = C(R(1)+1:end);%the detail coefficients
   NZ2(SNR,k) = length(X(X~=0));
   
   %wavelet denoising using SURE soft thresholding 
   [Wave_SURES,C,R] = wden(signal,'rigrsure','s','sln',levels,'sym4');
    
   X = C(R(1)+1:end);%the detail coefficients
   NZ3(SNR,k) = length(X(X~=0));

   %translation invariant hard wavelet thresholding, the filter used is
   %symlet 4
   Wave_TI = ThreshWave(signal,'H',1,0,sqrt(2*log(L)),levels); 
   
   %wavelet denoising using Bayes soft thresholding 
   [Wave_BayesS,C,R] = BayesShrink(signal,levels,'sym4');
   X = C(R(1)+1:end);%the detail coefficients
   NZ4(SNR,k) = length(X(X~=0));

   outReconNL(SNR,k)   = 10*log10(sigpower/mean((s-ReconNL).^2));
   outReconNLS(SNR,k)  = 10*log10(sigpower/mean((s-ReconNLS).^2));
   outWaveSUREH(SNR,k) = 10*log10(sigpower/mean((s-Wave_SUREH).^2));
   outWaveSURES(SNR,k) = 10*log10(sigpower/mean((s-Wave_SURES).^2));
   outWaveTI(SNR,k)    = 10*log10(sigpower/mean((s-Wave_TI).^2));
   outWaveBayesS(SNR,k)= 10*log10(sigpower/mean((s-Wave_BayesS).^2));
  end
 end
 
 %average non zero coefficients over all the realizations
 outNZ             = mean(NZ,2);
 outNZ1            = mean(NZ1,2);
 outNZ2            = mean(NZ2,2);
 outNZ3            = mean(NZ3,2);
 outNZ4            = mean(NZ4,2);

 %average reconstruction error (in dB) over all the realizations 
 outReconNLfin     = mean(outReconNL,2);
 outReconNLSfin    = mean(outReconNLS,2);
 outWaveSUREHfin   = mean(outWaveSUREH,2);
 outWaveSURESfin   = mean(outWaveSURES,2);
 outWaveTIfin      = mean(outWaveTI,2);
 outWaveBayesSfin  = mean(outWaveBayesS,2);
   
